/*
 * CohenSutherland.java
 *
 * Created on Tuesday, 2004, september 28, 11:21
 */

package cz.cuni.jagrlib.testing;

import cz.cuni.jagrlib.*;
import cz.cuni.jagrlib.iface.*;
import cz.cuni.jagrlib.piece.*;
import cz.cuni.jagrlib.reg.*;

/**
 *
 * @author  Miroslav Tynovsky
 */

public class CohenSutherland extends Piece implements RectangleWindow, LineRender {
    
    private int xMini, xMaxi, yMini, yMaxi;
    
    /**
     * Draws a line from [x1,y1] to [x2,y2].
     *  @param  x1 X coordinate of the starting pixel.
     *  @param  y1 Y coordinate of the starting pixel.
     *  @param  x2 X coordinate of the line end.
     *  @param  y2 Y coordinate of the line end.
     */
    public void drawLine(double x1, double y1, double x2, double y2) {
        this.drawLine((int)x1, (int)y1, (int)x2, (int)y2);
    }
    
    /**
     * Draws a line from [x1,y1] to [x2,y2].
     *  @param  x1 X coordinate of the starting pixel.
     *  @param  y1 Y coordinate of the starting pixel.
     *  @param  x2 X coordinate of the line end.
     *  @param  y2 Y coordinate of the line end.
     */
    public void drawLine(int x1, int y1, int x2, int y2) {
        LineRender line = (LineRender)getInterface(PL_OUTPUT,IFACE+"LineRender");

        /* 2) cela uvnitr okna? */
        if((kod(x1, y1) | kod(x2, y2)) == 0) {
            line.drawLine(x1, y1, x2, y2);
            return;
        }
        
        /* 3) cela mimo okno? */
        if((kod(x1, y1) & kod(x2, y2)) != 0){
            return;
        }
        
        /* 4) orez */
        int xx, yy;
        xx= xMini;
        yy= y1 + (y2-y1)*(xMini-x1)/(x2-x1);
        if ((kod(x1,y1) % 2)==1){ x1= xx; y1= yy; }
        if ((kod(x2,y2) % 2)==1){ x2= xx; y2= yy; }
        
        xx= xMaxi;
        yy= y1 + (y2-y1)*(xMaxi-x1)/(x2-x1);
        if (((kod(x1,y1)/2) % 2)==1){ x1= xx; y1= yy; }
        if (((kod(x2,y2)/2) % 2)==1){ x2= xx; y2= yy; }
        
        /* neco > yMaxi */
        xx= x1 + (x2-x1)*(yMaxi-y1)/(y2-y1);
        yy= yMaxi;
        if (((kod(x1,y1)/4) % 2)==1){ x1= xx; y1= yy; }
        if (((kod(x2,y2)/4) % 2)==1){ x2= xx; y2= yy; }
        
        /* neco < yMini */
        xx= x1 + (x2-x1)*(yMini-y1)/(y2-y1);
        yy= yMini;
        if (((kod(x1,y1)/8) % 2)==1){ x1= xx; y1= yy; }
        if (((kod(x2,y2)/8) % 2)==1){ x2= xx; y2= yy; }
        
        line.drawLine(x1, y1, x2, y2);
    }
    
    /**
     * Sets the given rectangle as clipping window.
     *  @param  xMin Minimal x coordinate.
     *  @param  xMax Maximal x coordinate.
     *  @param  yMin Minimal y coordinate.
     *  @param  yMax Maximal y coordinate.
     */
    public void rectangleWindow(double xMin, double xMax, double yMin, double yMax) {
        xMini= (int)xMin;
        xMaxi= (int)xMax;
        yMini= (int)yMin;
        yMaxi= (int)yMax;
    }
    
    /**
     * Sets the given rectangle as clipping window.
     *  @param  xMin Minimal x coordinate.
     *  @param  xMax Maximal x coordinate.
     *  @param  yMin Minimal y coordinate.
     *  @param  yMax Maximal y coordinate.
     */
    public void rectangleWindow(int xMin, int xMax, int yMin, int yMax) {
        xMini= xMin;
        xMaxi= xMax;
        yMini= yMin;
        yMaxi= yMax;
    }
    
    private int kod(int x, int y)
    {
        int oooo= 0,  oloo= 4,  looo=  8,  lloo= 12,
            oool= 1,  olol= 5,  lool=  9,  llol= 13,
            oolo= 2,  ollo= 6,  lolo= 10,  lllo= 14,
            ooll= 3,  olll= 7,  loll= 11,  llll= 15;
        
        int kod= 0;
        
        if(x < xMini) kod= kod | oool;
        if(x > xMaxi) kod= kod | oolo;
        if(y < yMini) kod= kod | looo;
        if(y > yMaxi) kod= kod | oloo;
        
        return kod;
    }
        
  //==========================================================================
  //  Module registration:

  /** Object name. */
  private final static String NAME = "CohenSutherland";

  /** Object template identifier. */
  protected final static String TEMPLATE_NAME = "CohenSutherland";

  /** Object description. */
  private final static String DESCRIPTION = "Draws and clips a line. ";

  /** Object category. */
  protected final static String CATEGORY = C_2D + "." + C_DRAW + "." + C_LINE;

  /**
   *  General-purpose registration routine.
   *  Sets all plugs, strings, etc. to the given Template.
   */
  public static int setTemplate ( Template t, int ord )
  {
    if ( t != null && ord <= 0 )
    {
      t.setRegStrings(NAME,TEMPLATE_NAME,CATEGORY,DESCRIPTION);
        // Plugs:
      t.newInputPlug(PL_INPUT+"Line",IFACE+"LineRender");
      t.newInputPlug(PL_INPUT+"Window",IFACE+"RectangleWindow");
      
      t.newOutputPlug(PL_OUTPUT,IFACE+"LineRender");
    }
    return 1;
  }
  
  /**
   *  Static registration instance for this class.
   *  Automatically initialized in class-loading time.
   */
  public static final RegPiece reg;
  static
  {
    reg = new RegPiece();
    setTemplate(reg,0);
  }   
}