/*
 * CohenSutherlandWorker.java
 *
 * Created on 30. z 2004, 3:37
 */

package cz.cuni.jagrlib.testing;

import cz.cuni.jagrlib.*;
import cz.cuni.jagrlib.iface.*;
import cz.cuni.jagrlib.piece.*;
import cz.cuni.jagrlib.reg.*;

/**
 *
 * @author  Miroslav Tynovsky
 */

public class CohenSutherlandWorker extends Piece implements Worker {
    
    /** Creates a new instance of CohenSutherlandWorker */
    public CohenSutherlandWorker() {
    }
    

    public float progress(float[] stages) {
        return 1.0f;
    }
    
    public void run() {
        LineRender line_full = (LineRender)getInterface(PL_OUTPUT+"Direct",IFACE+"LineRender");
        LineRender line_clipped = (LineRender)getInterface(PL_OUTPUT+"Clipped",IFACE+"LineRender");
        RectangleWindow wnd = (RectangleWindow)getInterface("RectWindow",IFACE+"RectangleWindow");
        SolidPen pen = (SolidPen)getInterface(PL_BITMASK,IFACE+"SolidPen");
        final GraphicsViewer preview = (GraphicsViewer)getInterface(PL_RASTER,IFACE+"GraphicsViewer");
        DataFileFormat file = (DataFileFormat)getInterface(PL_TRIGGER,IFACE+"DataFileFormat");
        
        if ( preview != null ) {
          Semaphore sem = new Semaphore(0);
          preview.addSemaphore(sem);
          Thread runIt = new Thread()    {
            public void run () { 
                preview.repaintLoop(); 
            }
          };
          try {
            runIt.setPriority(Thread.MIN_PRIORITY);
          } catch ( Exception e ) {
            LogFile.exception("Cannot set priority of preview thread! "+e.getMessage(),e);
          }
          try {
            runIt.start();
          } catch ( IllegalThreadStateException e ) {
            LogFile.exception("Cannot start preview thread! "+e.getMessage(),e);
          }
        }
        
        int[] color = new int[3];
        
        //draw full lines
        int x0, y0, x1, y1, x2, y2, x3, y3;
        x0= 30;    y0= 30;
        x1= 150;   y1= 400;
        x2= 300;   y2= 50;
        x3= 500;   y3= 200;
        
/*        line_full.drawLine(x0, y0, x1, y1);
        line_full.drawLine(x2, y2, x3, y3);
        color[0] = 0; color[1]= 255; color[2]= 0;
        pen.setColor(color);
        pen.stroke();
*/        
        //draw and set clipping window
        int wndx1, wndy1, wndx2, wndy2;
        wndx1= 80;  wndy1=150;
        wndx2= 450; wndy2= 300;
        line_full.drawLine(wndx1, wndy1, wndx2, wndy1);
        line_full.drawLine(wndx1, wndy1, wndx1, wndy2);
        line_full.drawLine(wndx2, wndy2, wndx1, wndy2);
        line_full.drawLine(wndx2, wndy2, wndx2, wndy1);
        color[0] = 255; color[1]= 0; color[2]= 0;
        pen.setColor(color);
        pen.stroke();
        wnd.rectangleWindow(wndx1, wndx2, wndy1, wndy2);
        
        //draw the clipped lines
//        line_clipped.drawLine(x0+5, y0+5, x1+5, y1+5);
//        line_clipped.drawLine(x2+5, y2+5, x3+5, y3+5);
        color[0] = 255; color[1]= 255; color[2]= 255;
        pen.setColor(color);
        pen.stroke();
//            if ( line == null || pen == null ) return;
//    int[] color = new int[] { 255, 255, 255, 255 };

      // draw random lines:
    int repeat=20;
    for ( int i = 0; i++ < repeat; /*stage++ */)
    {
        // random color:
      color[0] = 54 + (int)(Math.random()*200.0);
      color[1] = 54 + (int)(Math.random()*200.0);
      color[2] = 54 + (int)(Math.random()*200.0);
      pen.setColor(color);
    try
      {
          // random line:
        line_clipped.drawLine(Math.random()*511.0,Math.random()*511.0,
                      Math.random()*511.0,Math.random()*511.0);
          // stroke it:
        if ( (i & 15) == 0 ) pen.stroke();
          // refresh the output window:
        if ( preview != null && (i & 127) == 0 )
        {
          preview.repaintAll();
          preview.set(GraphicsViewer.WINDOW_TITLE,"Output Image - "+i+" lines");
        }
      }
      catch ( JaGrLibException e )
      {
        break;
      }
    }
        
        // finish repainting:
/*        if ( preview != null )
        {
          preview.repaintAll();
          preview.set(GraphicsViewer.WINDOW_TITLE,"Output Image - FINISHED");
          preview.stopRepaintLoop();
        }
*/    
    } /* run() */

  //==========================================================================
  //  Module registration:

  /** Object name. */
  private final static String NAME = "CohenSutherlandWorker";

  /** Object template identifier. */
  protected final static String TEMPLATE_NAME = "CohenSutherlandWorker";

  /** Object description. */
  private final static String DESCRIPTION = "worker for CS clipping algorithm";

  /** Object category. */
  protected final static String CATEGORY = C_2D + "." + C_DRAW + "." + C_LINE + "." + C_WORKER;

  /**
   *  General-purpose registration routine.
   *  Sets all plugs, strings, etc. to the given Template.
   */
  public static int setTemplate ( Template t, int ord )
  {
    if ( t != null && ord <= 0 )
    {
      t.setRegStrings(NAME,TEMPLATE_NAME,CATEGORY,DESCRIPTION);
        // Plugs:
      t.newOutputPlug(PL_OUTPUT+"Direct",IFACE+"LineRender");
      t.newOutputPlug(PL_OUTPUT+"Clipped",IFACE+"LineRender");
      t.newOutputPlug("RectWindow", IFACE+"RectangleWindow");
      t.newOutputPlug(PL_BITMASK,IFACE+"SolidPen");
      t.newOutputPlug(PL_RASTER,IFACE+"GraphicsViewer");
      t.newOutputPlug(PL_TRIGGER,IFACE+"DataFileFormat");
      
      
    }
    return 1;
  }

  /**
   *  Static registration instance for this class.
   *  Automatically initialized in class-loading time.
   */
  public static final RegPiece reg;
  static
  {
    reg = new RegPiece();
    setTemplate(reg,0);
  }    
}
